using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.Windows.Forms;
using namespace_PADLL_DOT_NET;  // Remeber we need PADLL.DLL as well
using CSGetStatusOnlyLMmisc;
using System.Drawing;
using System.Drawing.Printing;
//using PAPrinterClass;  // Use PAPrinterClass (No RESET_DC between pages)

namespace CSGetStatusOnlyLM
{

    public partial class CSGetStatusOnlyForm : Form
    {

        public string SelectedPrinter;

        public CSGetStatusOnlyForm()
        {
            InitializeComponent();
        }

        private void cmdGetStatus_Click(object sender, EventArgs e)
        {
            GetStatusGeneral();
        }

        public void GetStatusGeneral()
        {
            int StatusType;
            int lResult;
            int i;
            string StringStatusBytesHex;
            string StringStatusBytesAscii;

            // Create the StatusBuffer byte array 
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] StatusBytesRead = new Int32[1];
            Int32 StatusBytesReadInt;

            Clear_Fields();
            // Get status via Language Monitor 
            StatusType = PAConstants.PA_STAT_SHORT;
            if (bShort.Checked) StatusType = PAConstants.PA_STAT_SHORT;
            if (bNormal.Checked) StatusType = PAConstants.PA_STAT_NORMAL;
            if (bExtended.Checked) StatusType = PAConstants.PA_STAT_EXTENDED;
            if (bTicketCount.Checked) StatusType = PAConstants.PA_STAT_COUNT;
            if (bPHTemp.Checked) StatusType = PAConstants.PA_STAT_TEMPERATURE;
            if (bComplete.Checked) StatusType = PAConstants.PA_STAT_COMPLETE;

            // Display the type of status to get 
            switch (StatusType)
            {
                case PAConstants.PA_STAT_SHORT:
                    txtRawStatus.Text = "Short Status Pending ................";
                    break;
                case PAConstants.PA_STAT_NORMAL:
                    txtRawStatus.Text = "Normal Status Pending ................";
                    break;
                case PAConstants.PA_STAT_EXTENDED:
                    txtRawStatus.Text = "Extended Status Pending ................";
                    break;
                case PAConstants.PA_STAT_COUNT:
                    txtRawStatus.Text = "Ticket Count Pending ................";
                    break;
                case PAConstants.PA_STAT_TEMPERATURE:
                    txtRawStatus.Text = "Print Head temperature Pending ................";
                    break;
                case PAConstants.PA_STAT_FORMLENGTH:
                    txtRawStatus.Text = "Forms length Pending ................";
                    break;
                case PAConstants.PA_STAT_COMPLETE:
                    txtRawStatus.Text = "Complete Pending ................";
                    break;
                default:
                    txtRawStatus.Text = "Illegal Status request ................";
                    return; // TODO: might not be correct. Was : Exit Sub 
            }

            Application.DoEvents();

            // The actual status call 
            lResult = paUtils.paLMGetStatus(StatusType, StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);
            // Move the number of bytes read to a conveninent integer variable
            StatusBytesReadInt = StatusBytesRead[0];
            // txtReturnCode.Text = paFunctions.FormatReturnCode(lResult) 

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                StringStatusBytesHex = "";
                StringStatusBytesAscii = "";
                for (i = 0; i <= StatusBytesReadInt - 1; i++)
                {
                    StringStatusBytesHex = StringStatusBytesHex + String.Format("{0:x2} ", StatusBuffer[i]);
                    if (StatusBuffer[i] > 0)
                        StringStatusBytesAscii += Convert.ToChar(StatusBuffer[i]);
                    else
                        StringStatusBytesAscii += '.';
                }
                txtRawStatus.Text = StringStatusBytesHex;
                txtAsciiStatus.Text = StringStatusBytesAscii;

                // Parse the status and display 
                ParseStatus ps = new ParseStatus();
                lResult = paUtils.paParseStatus(StatusBuffer, StatusBytesReadInt, ref ps);

                txtVersion.Text = ps.ver;
                txtCode.Text = ps.type;
                txtBits.Text = ps.chStatBits;
                txtTemp.Text = ps.chTemperature;
                txtCount.Text = ps.chDocCount;
                txtErrorCode.Text = ps.chError;
                txtLength.Text = ps.chFormLength;

                if ((ps.type != "A"))
                {
                    if (((ps.lStatBits & PAConstants.STAT16_SYSTEM_NOT_READY) == 0))
                        txtReady.Text = "Printer Ready";
                    else
                        txtReady.Text = "Printer Not Ready";
                }
            }
            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void CSGetStatusOnlyForm_Load(object sender, EventArgs e)
        {
            Int32 lResult = 0;

            if (paUtils.paLMSetPipeTimeOut(PAConstants.DEFAULT_PIPE_TIMEOUT) == false)
                MessageBox.Show("Could not set default pipe timeout");

            if (paUtils.paLMSetStatusTimeOut(PAConstants.DEFAULT_STATUS_TIMEOUT) == false)
                MessageBox.Show("Could not set default status timeout");

            txtTimeOut.Text = Convert.ToString(PAConstants.DEFAULT_STATUS_TIMEOUT);
            // Display the current printer name 

            // Because we are going to call a pa function that receives a pointer 
            // to a string and fills in a value we have to use the StringBuilder 
            // class 
            StringBuilder DefaultPrinterName = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);
            StringBuilder PrinterPort = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);
            StringBuilder PrinterDriver = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] SizeOfPrinterNameBuffer = new Int32[1];
            SizeOfPrinterNameBuffer[0] = PAConstants.PA_MAX_BUFFER_SIZE;
            lResult = paUtils.paNewGetDefaultPrinter(DefaultPrinterName, SizeOfPrinterNameBuffer);

            if (lResult == PAConstants.PA_ERR_OK)
            {
                txtPrinterName.Text = DefaultPrinterName.ToString();
                SelectedPrinter = txtPrinterName.Text;
                //temp start 
                lResult = paUtils.paNewGetPrinterPort(DefaultPrinterName.ToString(), PrinterPort);
                if (lResult == PAConstants.PA_ERR_OK)
                {
                    txtRawStatus.Text = PrinterPort.ToString();
                }

                lResult = paUtils.paNewGetPrinterDriver(DefaultPrinterName.ToString(), PrinterDriver);
                if (lResult == PAConstants.PA_ERR_OK)
                {
                    txtAsciiStatus.Text = PrinterDriver.ToString();
                }
            }
            else
            {
                txtPrinterName.Text = "Default Printer not found";
            }

            bShort.Checked = true;

            bOrientPortrait.Checked = true;

            bCutAll.Checked = true;

            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            // Populate the combo box for the LM set registry settings function 
            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            cmbKeyList.Items.Add("DeleteAllJobsOnStart");
            cmbKeyList.Items.Add("DRWdebugMask");
            cmbKeyList.Items.Add("DRWenable");
            cmbKeyList.Items.Add("DRWMaxFileSize");
            cmbKeyList.Items.Add("DRWMinFileSize");
            cmbKeyList.Items.Add("DRWpollTime");
            cmbKeyList.Items.Add("ECPmode");
            cmbKeyList.Items.Add("EnableDiags");
            cmbKeyList.Items.Add("EnableHeartBeat");
            cmbKeyList.Items.Add("EOJEjectSendLastPage");
            cmbKeyList.Items.Add("EOJTimeout");

            cmbKeyList.Items.Add("HeartBeatRate");
            cmbKeyList.Items.Add("JAVAlowPaperDisable");
            cmbKeyList.Items.Add("JAVAticketNotTakenDisable");
            cmbKeyList.Items.Add("LPmode");
            cmbKeyList.Items.Add("ParallelPacketSize");
            cmbKeyList.Items.Add("SerialPacketSize");
            cmbKeyList.Items.Add("USBAbortOnWriteTimeout");
            cmbKeyList.Items.Add("USBPacketSize");
            cmbKeyList.Items.Add("USBprintingTimeout");
            cmbKeyList.Items.Add("USBReadIntervalTimeout");

            cmbKeyList.Items.Add("USBReadTotalTimeoutConstant");
            cmbKeyList.Items.Add("USBReadTotalTimeoutMultiplier");
            cmbKeyList.Items.Add("USBWDGlobalEnable");
            cmbKeyList.Items.Add("USBWDtimeout");
            cmbKeyList.Items.Add("USBWDResetPrinterOnCRCError");
            cmbKeyList.Items.Add("USBWriteTotalTimeoutConstant");
            cmbKeyList.Items.Add("USBWriteTotalTimeoutMultiplier");
            cmbKeyList.Items.Add("USBWriteTotalTimeoutMultiplierFlashUpdate");

            // IBM Canada 09/18/2007 
            cmbKeyList.Items.Add("LPLatchEnable");
            cmbKeyList.Items.Add("LPLatched");
            cmbKeyList.Items.Add("LPLatchThreshold");
            cmbKeyList.Items.Add("LPLatchCount");

            cmbKeyList.Items.Add("BadValue");

            txtKeyValue.Text = "1";

            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            // Populate the combo box for the Printer set registry settings function 
            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            cmbPrinterKeyList.Items.Add("ROTATE_IMAGE_180");
            cmbPrinterKeyList.Items.Add("AUTOSIZE");
            cmbPrinterKeyList.Items.Add("BACKUP_CONTROL");
            cmbPrinterKeyList.Items.Add("BEEPER_CONTROL");
            cmbPrinterKeyList.Items.Add("DO_NOT_COMPRESS");
            cmbPrinterKeyList.Items.Add("ENERGY_LEVEL");
            cmbPrinterKeyList.Items.Add("ENERGY_TABLE");
            cmbPrinterKeyList.Items.Add("EOD_CONTROL");
            cmbPrinterKeyList.Items.Add("JAM_ERROR_CONTROL");
            cmbPrinterKeyList.Items.Add("LOCKED");
            cmbPrinterKeyList.Items.Add("LOW_PAPER_ACTIVE_DETECT");
            cmbPrinterKeyList.Items.Add("LOW_PAPER_ALGORITHMN");
            cmbPrinterKeyList.Items.Add("MIN_FORM_LENGTH");
            cmbPrinterKeyList.Items.Add("SJO_CONFIG");
            cmbPrinterKeyList.Items.Add("TNT_CONTROL");
            cmbPrinterKeyList.Items.Add("TNT_DISPOSE");
            cmbPrinterKeyList.Items.Add("TNT_EJECT_ALWAYS");
            cmbPrinterKeyList.Items.Add("USE_IPS_LIMIT");

            cmbPrinterKeyList.Items.Add("BadValue");

            txtPrinterKeyValue.Text = "1"; 

        }

        private void cmdGetHeadLeverStatus_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetHeadLeverOpenStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Print head lever is up";
                else
                    txtAsciiStatus.Text = "Print head lever is down";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }
        private void cmdGetOutOfPaperStatus_Click(object sender, EventArgs e)
        {

            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetOutOfPaperStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Printer is out of paper (tickets)";
                else
                    txtAsciiStatus.Text = "Print has paper (tickets)";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdGetLowPaperStatus_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetLowPaperStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Paper supply is low";
                else
                    txtAsciiStatus.Text = "Paper supply is not low";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void cmdGetManDeselectStatus_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetManDeselectStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Printer is Deselected";
                else
                    txtAsciiStatus.Text = "Printer is Selected";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdGetJamStatus_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetJamStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Printer is Jammed";
                else
                    txtAsciiStatus.Text = "Printer is NOT jammed";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdGetTNTStatus_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetTicketNotTakenStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Document in transporter";
                else
                    txtAsciiStatus.Text = "No document in transporter";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdPrinterOK_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetPrinterOKStatus(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Printer is ready to print";
                else
                    txtAsciiStatus.Text = "Printer has problem, cannot print";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void cmdGetAddressedStatus_Click(object sender, EventArgs e)
        {

            string StatusType;
            int lResult;
            int i;
            int StatusBytesReadInt;
            string StringStatusBytesHex;
            string StringStatusBytesAscii;

            // Create the StatusBuffer byte array 
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] StatusBytesRead = new Int32[1];

            Clear_Fields();
            // Get addressed status via Language Monitor 
            txtRawStatus.Text = "Getting addressed status Status request ................";

            Application.DoEvents();

            // The actual status call 
            StatusType = "\x1b";
            StatusType += "I";
            StatusType += "A";
            StatusType += "\x05";
            // Version number 
            //StatusType += Chr(8) ' Pnp String 

            lResult = paUtils.paLMGetAddStatus(StatusType, StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);
            // Move the number of bytes read to a convenient integer variable
            StatusBytesReadInt = StatusBytesRead[0];

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                StringStatusBytesHex = "";
                StringStatusBytesAscii = "";
                for (i = 0; i <= StatusBytesReadInt - 1; i++)
                {
                    StringStatusBytesHex = StringStatusBytesHex + String.Format("{0:x2} ", StatusBuffer[i]);
                    if (StatusBuffer[i] > 0)
                        StringStatusBytesAscii += Convert.ToChar(StatusBuffer[i]);
                    else
                        StringStatusBytesAscii += '.';
                }
                txtRawStatus.Text = StringStatusBytesHex;
                txtAsciiStatus.Text = StringStatusBytesAscii;

                // Parse the status and display 
                ParseStatus ps = new ParseStatus();
                lResult = paUtils.paParseStatus(StatusBuffer, StatusBytesReadInt, ref ps);

                txtVersion.Text = ps.ver;
                txtCode.Text = ps.type;
                txtBits.Text = ps.chStatBits;
                txtTemp.Text = ps.chTemperature;
                txtCount.Text = ps.chDocCount;
                txtErrorCode.Text = ps.chError;
                txtLength.Text = ps.chFormLength;

                if ((ps.type != "A"))
                {
                    if (((ps.lStatBits & 32768) == 0))
                        txtReady.Text = "Printer Ready";
                    else
                        txtReady.Text = "Printer Not Ready";
                }
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdGetPrinterName_Click(object sender, EventArgs e)
        {
            StringBuilder PrinterName = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);
            Int32[] SizeOfPrinterNameBuffer = new Int32[1];

            Clear_Fields();
            SizeOfPrinterNameBuffer[0] = PAConstants.PA_MAX_BUFFER_SIZE;
            paUtils.paNewGetDefaultPrinter(PrinterName, SizeOfPrinterNameBuffer);
            txtAsciiStatus.Text = PrinterName.ToString();
        }

        private void cmdGetProgress_Click(object sender, EventArgs e)
        {
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];
            Int32[] StatusBytesRead = new Int32[1];
            string ResultString;

            paUtils.paLMGetProgress("O", StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);
            // Convert the result to a string 
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            ResultString = enc.GetString(StatusBuffer);

            flashProgress fp = new flashProgress();

            paUtils.paLMParseFlashProgress(ResultString, ref fp);
            MessageBox.Show(String.Format("{0:x1},{1:x1},{2:x1},{3},{4} ", fp.state, fp.max, fp.current, fp.PassFail, fp.note));

        }

        private void cmdGetDriverVersion_Click(object sender, EventArgs e)
        {
            StringBuilder PrinterVersion = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);
            Int32[] VersionBytesRead = new Int32[1];

            Clear_Fields();
            paUtils.paLMGetDriverVersion(SelectedPrinter, PrinterVersion, PAConstants.PA_MAX_BUFFER_SIZE, VersionBytesRead);
            txtAsciiStatus.Text = PrinterVersion.ToString();
        }

        private void cmdGetPortName_Click(object sender, EventArgs e)
        {
            StringBuilder PortName = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);

            Clear_Fields();
            paUtils.paNewGetPrinterPort(SelectedPrinter, PortName);
            txtAsciiStatus.Text = PortName.ToString();
        }

        private void cmdGetPrintInPrc_Click(object sender, EventArgs e)
        {
            Int32 lResult = 0;
            SimpleStatusBool SimpleStatus = new SimpleStatusBool();

            Clear_Fields();
            lResult = paUtils.paLMGetPrintInProgress(SimpleStatus);
            if (lResult == PAConstants.PA_ERR_OK)
            {
                if (SimpleStatus.MarshalledSimpleStatus == true)
                    txtAsciiStatus.Text = "Printing is in progress";
                else
                    txtAsciiStatus.Text = "Printing is not in progress";
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }

        }

        private void cmdGetDLLVersion_Click(object sender, EventArgs e)
        {
            // Create the StatusBuffer byte array 
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] StatusBytesRead = new Int32[1];

            int i;
            string DLLVersion;

            paUtils.paGetDLLVersion(StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);

            // Convert the data from byte to char 
            DLLVersion = "";
            for (i = 0; i < StatusBytesRead[0]; i++)
            {
                DLLVersion += Convert.ToChar(StatusBuffer[i]);
            }

            txtAsciiStatus.Text = DLLVersion;
        }

        private void cmdGetLMVersion_Click(object sender, EventArgs e)
        {
            // Create the StatusBuffer byte array 
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] StatusBytesRead = new Int32[1];

            int i;
            string LMVersion;

            paUtils.paLMGetLMVersion(StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);

            // Convert the data from byte to char 
            LMVersion = "";
            for (i = 0; i < StatusBytesRead[0]; i++)
                LMVersion += Convert.ToChar(StatusBuffer[i]);

            txtAsciiStatus.Text = LMVersion;
        }

        private void cmdStartTimer_Click(object sender, EventArgs e)
        {
            timer1.Enabled = true;
            timer1.Interval = 1000;
            txtTimedCount.Text = "1";
            GetStatusGeneral();
        }

        private void cmdStopTimer_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            String sTempCount;
            int iTempCount;

            GetStatusGeneral();

            sTempCount = txtTimedCount.Text;
            iTempCount = Convert.ToInt32(sTempCount);
            iTempCount++;
            sTempCount = iTempCount.ToString();
            txtTimedCount.Text = sTempCount;
        }

        private void cmsSetNewPrinter_Click(object sender, EventArgs e)
        {
            bool bRc;

            Clear_Fields();
            bRc = paUtils.paSetNewPrinter(txtNewPrinter.Text);

            if (bRc == true)
            {
                txtAsciiStatus.Text = "Set printer name passed";
                SelectedPrinter = txtNewPrinter.Text;
                txtPrinterName.Text = SelectedPrinter;
            }
            else
            {
                txtAsciiStatus.Text = "Set printer name FAILED";
            }
        }

        private void cmdSetRegistryKey_Click(object sender, EventArgs e)
        {
            StringBuilder pResultString = new StringBuilder(PAConstants.PA_MAX_BUFFER_SIZE);
            Int32[] ResultBufferRead = new Int32[1];
            Int32 iKeyValue;

            Clear_Fields();
            iKeyValue = Convert.ToInt32(txtKeyValue.Text);
            paUtils.paLMSetRegistryKey(cmbKeyList.Text, iKeyValue, pResultString, PAConstants.PA_MAX_BUFFER_SIZE, ResultBufferRead);
            txtAsciiStatus.Text = pResultString.ToString(); 
        }

        private void cmdSetPrtRegistryKey_Click(object sender, EventArgs e)
        {
            int lResult;
            Int32 iKeyValue;

            Clear_Fields();
            iKeyValue = Convert.ToInt32(txtPrinterKeyValue.Text);
            lResult = paUtils.paSetPrinterKeyByName(cmbPrinterKeyList.Text, iKeyValue);

            if (lResult == PAConstants.PA_ERR_OK)
                txtAsciiStatus.Text = "Set Printer registry key success";
            else
                txtAsciiStatus.Text = paFunctions.FormatReturnCode(lResult);
        }

        private void cmdSetFormSizePrint_Click(object sender, EventArgs e)
        {
            paPrinter Print = new paPrinter();
            int MaxPageCount;

            MaxPageCount = 1;
            Print.print("Test Set Form Size", MaxPageCount, SelectedPrinter);

        }

        private void cmdSetFormSize_Click(object sender, EventArgs e)
        {
            float[] DesiredWidth = new float[1];
            float[] DesiredLength = new float[1];
            int lResult;

            Clear_Fields();
            Application.DoEvents();

            if (txtDesiredWidth.Text == "")
                DesiredWidth[0] = 8;
            else
                DesiredWidth[0] = Convert.ToSingle(txtDesiredWidth.Text);

            if (txtDesiredLength.Text == "")
                DesiredLength[0] = 8;
            else
                DesiredLength[0]  = Convert.ToSingle(txtDesiredLength.Text);

            lResult = paUtils.paSetSheetSize(DesiredWidth, DesiredLength);

            if (lResult == PAConstants.PA_ERR_OK)
                txtAsciiStatus.Text = "Exact form size set";
            else
                txtAsciiStatus.Text = paFunctions.FormatReturnCode(lResult);

            txtActualWidth.Text = Convert.ToString(DesiredWidth[0]);
            txtActualLength.Text = Convert.ToString(DesiredLength[0]); 
        }

        private void cmdSetOrientMode_Click(object sender, EventArgs e)
        {
            int lResult;

            Clear_Fields();
            if (bOrientPortrait.Checked)
                lResult = paUtils.paSetOrientation(1);
            else
                lResult = paUtils.paSetOrientation(2);

            if (lResult == PAConstants.PA_ERR_OK)
                txtAsciiStatus.Text = "Set Orientation Passed";
            else
                txtAsciiStatus.Text = paFunctions.FormatReturnCode(lResult);
        }

        private void cmdOrientPrint_Click(object sender, EventArgs e)
        {
            paPrinter Print = new paPrinter();
            int MaxPageCount;

            MaxPageCount = 1;
            if (bOrientPortrait.Checked)
                Print.print("Portrait mode", MaxPageCount, SelectedPrinter);
            else
                Print.print("Landscape Mode", MaxPageCount, SelectedPrinter);
        }

        private void cmdCutPacketTest_Click(object sender, EventArgs e)
        {
            paPrinter Print = new paPrinter();
            int MaxPageCount;
            int lResult;
            string CutCommand;

            CutCommand = PAConstants.PA_CUT_ALL;

            if (bCutAll.Checked) CutCommand = PAConstants.PA_CUT_ALL;
            if (bEvery2nd.Checked) CutCommand = PAConstants.PA_CUT_2ND;
            if (bEveryThird.Checked) CutCommand = PAConstants.PA_CUT_3RD;
            if (bEvery4th.Checked) CutCommand = PAConstants.PA_CUT_4TH;
            if (bEvery5th.Checked) CutCommand = PAConstants.PA_CUT_5TH;
            if (bCutLast.Checked) CutCommand = PAConstants.PA_CUT_LAST;
            if (bCutDisable.Checked) CutCommand = PAConstants.PA_NO_CUT;

            lResult = paUtils.paSetBinByBinName(CutCommand);
            if ((lResult == PAConstants.PA_ERR_OK))
            {
                txtAsciiStatus.Text = "Set bin by name passed";
                MaxPageCount = 3;
                Print.print("Cut Packeting Test", MaxPageCount, SelectedPrinter);
            }
            else
            {
                txtAsciiStatus.Text = paFunctions.FormatReturnCode(lResult);
            } 
        }

        private void DisplayRFIDStockStatus(byte[] DisplayData, int DataLength)
        {
            String strDisplayData = Encoding.ASCII.GetString(DisplayData, 0, DataLength);
            int iResult = 0;

            txtReady.Text = "";
            if (int.TryParse(strDisplayData.Substring(4, 3), out iResult) && (strDisplayData.Substring(7, 1) == ","))
            {
                iResult = Int32.Parse(strDisplayData.Substring(4, 3));
                if (iResult == 1)
                {
                    // Display Status
                    switch (DisplayData[8])
                    {
                        case PAConstants.RFID_READER_NOT_DETERMINED:
                            txtReady.Text = "RFID READER NOT DETERMINED";
                            break;

                        case PAConstants.RFID_NO_READER:
                            txtReady.Text = "RFID NO READER";
                            break;

                        case PAConstants.RFID_READER_NON_FUNCTIONAL:
                            txtReady.Text = "RFID READER NON FUNCTIONAL";
                            break;

                        case PAConstants.RFID_NO_STOCK:
                            txtReady.Text = "RFID NO STOCK";
                            break;

                        case PAConstants.RFID_TAGLESS_STOCK:
                            txtReady.Text = "RFID TAGLESS STOCK";
                            break;

                        case PAConstants.RFID_STOCK_LOADED:
                            txtReady.Text = "RFID STOCK LOADED";
                            break;

                        case PAConstants.RFID_TAG_READY:
                            txtReady.Text = "RFID TAG READY";
                            break;

                        case PAConstants.RFID_TAG_FAULTY:
                            txtReady.Text = "RFID TAG FAULTY";
                            break;
                    
                        default:
                            txtReady.Text = "RFID TAG STATUS UNKNOWN";
                            break;
                    }
                }
            }
        }

        private void DisplayRFIDStatus(byte[] DisplayData, int DataLength)
        {
            if (DisplayData[3] == 's')
            {
                // Stock Status
                DisplayRFIDStockStatus(DisplayData, DataLength);
                return;
            }

            String strDisplayData = Encoding.ASCII.GetString(DisplayData, 0, DataLength);
            int iResult = 0;

            txtReady.Text = "";
            if (int.TryParse(strDisplayData.Substring(4, 3), out iResult) && (strDisplayData.Substring(7, 1) == ","))
            {
                iResult = Int32.Parse(strDisplayData.Substring(4, 3));
                if (iResult == 1)
                {
                    // Display Status
                    switch (DisplayData[8])
                    {
                        case PAConstants.RFID_SUCCESS:
                            txtReady.Text = "RFID SUCCESS";
                            break;

                        case PAConstants.RFID_BUSY:
                            txtReady.Text = "RFID BUSY";
                            break;

                        case PAConstants.RFID_FAILURE:
                            txtReady.Text = "RFID FAILURE";
                            break;

                        case PAConstants.RFID_CANCELLING:
                            txtReady.Text = "RFID CANCELLING";
                            break;

                        case PAConstants.RFID_CANCELLED:
                            txtReady.Text = "RFID CANCELLED";
                            break;

                        case PAConstants.RFID_NO_STATUS:
                            txtReady.Text = "RFID NO STATUS";
                            break;

                        case PAConstants.RFID_INVALID_COMMAND:
                            txtReady.Text = "RFID INVALID COMMAND";
                            break;
                        
                        default:
                            txtReady.Text = "UNKNOWN";
                            break;
                    }
                }
            }
        }

        private void DisplayRawAsciiText(byte[] DisplayData, int DataLength)
        {
            string StringBytesHex = "";
            string StringBytesAscii = "";
            int Counter = 0;

            StringBytesHex = "";
            StringBytesAscii = "";
            for (Counter = 0; Counter < DataLength; Counter++)
            {
                StringBytesHex = StringBytesHex + String.Format("{0:x2} ", DisplayData[Counter]);
                if (DisplayData[Counter] > 0)
                    StringBytesAscii += Convert.ToChar(DisplayData[Counter]);
                else
                    StringBytesAscii += '.';
            }
            txtRawStatus.Text = StringBytesHex;
            txtAsciiStatus.Text = StringBytesAscii;
        }

        public void Clear_Fields()
        {
            txtVersion.Text = "";
            txtCode.Text = "";
            txtBits.Text = "";
            txtTemp.Text = "";
            txtCount.Text = "";
            txtErrorCode.Text = "";
            txtLength.Text = "";
            txtReady.Text = "";
            txtAsciiStatus.Text = "";
            txtRawStatus.Text = "";
        }

        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
        // Print test documents using the PrintDocument class. 
        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
        public class paPrinter
        {
            // Friend access means that the elements are accessible only withing 
            // the program. 
            internal string TextToBePrinted;
            internal int PageCount;
            internal int MaxPageCount;

            public void print(string text, int MaxCount, String CurrentlySelectedPrinter)
            {

                TextToBePrinted = text;
                MaxPageCount = MaxCount;
                // Dim prn As New Printing.PrintDocument 
                // If the following is included Imports System.Drawing.Printing 
                // you can simply do: Dim prn As New PrintDocument 
                PrintDocument prn = new PrintDocument();

                using ((prn))
                {
                    // prn.PrinterSettings.PrinterName is set to the default printer 
                    // If other than default you must: 
                    prn.PrinterSettings.PrinterName = CurrentlySelectedPrinter; 
                    //prn.PrinterSettings.PrinterName = "PA ITX3002G" 

                    // The following AddHandler attaches the PrintPageHandler event 
                    // to the print process. The PrintPageHandler will be called 
                    // as an event for each page. 
                    prn.PrintPage += this.PrintPageHandler;

                    // If we want to use the PAPrintController we must:
                    // 1 - Add a reference (PAPrinterClass)
                    // 2 - using PAPrinterClass;
                    // 3 - Add the following here
                    //prn.PrintController = new PrintControllerWithStatusDialog(new PAPrintController(), "Printing");
                    
                    PageCount = 0;
                    prn.Print();
                    prn.PrintPage -= this.PrintPageHandler;
                }
            }

            private void PrintPageHandler(object sender, System.Drawing.Printing.PrintPageEventArgs args)
            {
                Font myFont = new Font("Microsoft San Serif", 20);

                args.Graphics.DrawString(TextToBePrinted, new Font(myFont, FontStyle.Regular), Brushes.Black, 50, 50);

                PageCount = PageCount + 1;

                if (PageCount < MaxPageCount)
                {
                    args.HasMorePages = true;
                }
                else
                {
                    args.HasMorePages = false;
                }

            }
        }

        private void cmdReadRFIDStock_Click(object sender, EventArgs e)
        {
            int lResult = 0;
            int BytesReadInt = 0;
            // Create the StatusBuffer byte array 
            byte[] ResponseBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];
            byte[] DataToWrite = null;

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] BytesRead = new Int32[1];

            Clear_Fields();
            // Get addressed status via Language Monitor 
            txtRawStatus.Text = "RFID Command Pending ................";

            Application.DoEvents();

            // The actual status call 
            DataToWrite = Encoding.ASCII.GetBytes("\x1b" + "RFS000");

            lResult = paUtils.paLMRFIDPrinter(DataToWrite, DataToWrite.Length, ResponseBuffer, PAConstants.PA_MAX_BUFFER_SIZE, BytesRead, 500);
            // Move the number of bytes read to a convenient integer variable
            BytesReadInt = BytesRead[0];

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                // string strResponseString = Encoding.ASCII.GetString(ResponseBuffer, 0, BytesReadInt).ToString();
                DisplayRawAsciiText(ResponseBuffer, BytesReadInt);
                DisplayRFIDStatus(ResponseBuffer, BytesReadInt);
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void cmdRFIDCustomRequest_Click(object sender, EventArgs e)
        {
            String strCommandToSend = "";
            int BytesReadInt = 0;
            int lResult = 0;
            // Create the StatusBuffer byte array 
            byte[] ResponseBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];
            byte[] DataToWrite = null;

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] BytesRead = new Int32[1];

            Clear_Fields();
            // Get addressed status via Language Monitor 
            txtRawStatus.Text = "RFID Command Pending ................";

            Application.DoEvents();

            if (txtRFIDCustomRequest.Text.Length == 0)
            {
                txtRawStatus.Text = "No command to send";
                return;
            }

            strCommandToSend = "\x1b" + txtRFIDCustomRequest.Text;

            // The actual status call 
            DataToWrite = Encoding.ASCII.GetBytes(strCommandToSend);

            lResult = paUtils.paLMRFIDPrinter(DataToWrite, DataToWrite.Length, ResponseBuffer, PAConstants.PA_MAX_BUFFER_SIZE, BytesRead, 500);
            // Move the number of bytes read to a convenient integer variable
            BytesReadInt = BytesRead[0];

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                // string strResponseString = Encoding.ASCII.GetString(ResponseBuffer, 0, BytesReadInt).ToString();
                DisplayRawAsciiText(ResponseBuffer, BytesReadInt);
                DisplayRFIDStatus(ResponseBuffer, BytesReadInt);
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void cmdRFIDReadTag_Click(object sender, EventArgs e)
        {
            String strCommandToSend = "";
            int BytesReadInt = 0;
            int lResult = 0;
            // Create the StatusBuffer byte array 
            byte[] ResponseBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];
            byte[] DataToWrite = null;

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] BytesRead = new Int32[1];

            Clear_Fields();
            // Get addressed status via Language Monitor 
            txtRawStatus.Text = "RFID Command Pending ................";

            Application.DoEvents();

            if (txtRFIDReadTag.Text.Length == 0)
            {
                txtRawStatus.Text = "No read command to send";
                return;
            }

            strCommandToSend = "\x1b" + "RFR" + String.Format("{0:000}", txtRFIDReadTag.Text.Length) + "," + txtRFIDReadTag.Text;

            // The actual status call 
            DataToWrite = Encoding.ASCII.GetBytes(strCommandToSend);

            lResult = paUtils.paLMRFIDPrinter(DataToWrite, DataToWrite.Length, ResponseBuffer, PAConstants.PA_MAX_BUFFER_SIZE, BytesRead, 500);
            // Move the number of bytes read to a convenient integer variable
            BytesReadInt = BytesRead[0];

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                // string strResponseString = Encoding.ASCII.GetString(ResponseBuffer, 0, BytesReadInt).ToString();
                DisplayRawAsciiText(ResponseBuffer, BytesReadInt);
                DisplayRFIDStatus(ResponseBuffer, BytesReadInt);
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void cmdRFIDWriteTag_Click(object sender, EventArgs e)
        {
            String strCommandToSend = "";
            int BytesReadInt = 0;
            int lResult = 0;
            // Create the StatusBuffer byte array 
            byte[] ResponseBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];
            byte[] DataToWrite = null;

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] BytesRead = new Int32[1];

            Clear_Fields();
            // Get addressed status via Language Monitor 
            txtRawStatus.Text = "RFID Command Pending ................";

            Application.DoEvents();

            if (txtRFIDWriteTag.Text.Length == 0)
            {
                txtRawStatus.Text = "No write data to send";
                return;
            }

            strCommandToSend = "\x1b" + "RFW" + String.Format("{0:000}", txtRFIDWriteTag.Text.Length) + "," + txtRFIDWriteTag.Text;

            // The actual status call 
            DataToWrite = Encoding.ASCII.GetBytes(strCommandToSend);

            lResult = paUtils.paLMRFIDPrinter(DataToWrite, DataToWrite.Length, ResponseBuffer, PAConstants.PA_MAX_BUFFER_SIZE, BytesRead, 500);
            // Move the number of bytes read to a convenient integer variable
            BytesReadInt = BytesRead[0];

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                // string strResponseString = Encoding.ASCII.GetString(ResponseBuffer, 0, BytesReadInt).ToString();
                DisplayRawAsciiText(ResponseBuffer, BytesReadInt);
                DisplayRFIDStatus(ResponseBuffer, BytesReadInt);
            }
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        private void btnIsPtrPipe_Click(object sender, EventArgs e)
        {
            Clear_Fields();

            if (paUtils.paLMPtrIsPipe(txtNewPrinter.Text))
            {
                txtAsciiStatus.Text = "Printer has a valid pipe";
            }
            else
            {
                txtAsciiStatus.Text = "Printer does NOT have a valid pipe";
            }
        }

        private void btnGetPrinterCompleteStatus_Click(object sender, EventArgs e)
        {
            int lResult = 0;
            int i = 0;
            string StringStatusBytesHex = "";
            string StringStatusBytesAscii = "";

            // Create the StatusBuffer byte array 
            byte[] StatusBuffer = new byte[PAConstants.PA_MAX_BUFFER_SIZE];

            // Create the DWORD(Int32) value for returning the number of bytes 
            // read. Note that Marshalling requires we create an array of 1 element. 
            Int32[] StatusBytesRead = new Int32[1];
            Int32 StatusBytesReadInt = 0;

            Clear_Fields();

            // Get status via Language Monitor 
            txtRawStatus.Text = "Complete Pending ................"; ;

            Application.DoEvents();

            // The actual status call 
            lResult = paUtils.paLMPtrGetStatus(txtNewPrinter.Text, PAConstants.PA_STAT_COMPLETE, StatusBuffer, PAConstants.PA_MAX_BUFFER_SIZE, StatusBytesRead);
            // Move the number of bytes read to a conveninent integer variable
            StatusBytesReadInt = StatusBytesRead[0];
            // txtReturnCode.Text = paFunctions.FormatReturnCode(lResult) 

            if (lResult == PAConstants.PA_ERR_OK)
            {
                // Good com with printer so display the result 

                // Convert to HEX and Ascii display 
                StringStatusBytesHex = "";
                StringStatusBytesAscii = "";
                for (i = 0; i <= StatusBytesReadInt - 1; i++)
                {
                    StringStatusBytesHex = StringStatusBytesHex + String.Format("{0:x2} ", StatusBuffer[i]);
                    if (StatusBuffer[i] > 0)
                        StringStatusBytesAscii += Convert.ToChar(StatusBuffer[i]);
                    else
                        StringStatusBytesAscii += '.';
                }
                txtRawStatus.Text = StringStatusBytesHex;
                txtAsciiStatus.Text = StringStatusBytesAscii;

                // Parse the status and display 
                ParseStatus ps = new ParseStatus();
                lResult = paUtils.paParseStatus(StatusBuffer, StatusBytesReadInt, ref ps);

                txtVersion.Text = ps.ver;
                txtCode.Text = ps.type;
                txtBits.Text = ps.chStatBits;
                txtTemp.Text = ps.chTemperature;
                txtCount.Text = ps.chDocCount;
                txtErrorCode.Text = ps.chError;
                txtLength.Text = ps.chFormLength;

                if ((ps.type != "A"))
                {
                    if (((ps.lStatBits & PAConstants.STAT16_SYSTEM_NOT_READY) == 0))
                        txtReady.Text = "Printer Ready";
                    else
                        txtReady.Text = "Printer Not Ready";
                }
            }
            ///'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
            else
            {
                MessageBox.Show(MiscClass.GetReturnCodeString(lResult));
            }
        }

        ///////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////
    }

}


