using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


namespace namespace_PADLL_DOT_NET
{
    [StructLayout(LayoutKind.Sequential)]
    public class SimpleStatusBool
    {
        [MarshalAs(UnmanagedType.Bool)]
        public bool MarshalledSimpleStatus;
    }

    [StructLayout(LayoutKind.Sequential)]
    public class flashProgress
    {
        public long state;         // 0 = initial delay, 1 = sending data, 2 = end delay, 3 = complete
        public long max;           // Maximum value for this state
        public long current;       // Current value for this state
        public string PassFail;    // Pass 'P' or Fail 'F'
        public string note;        // Reason for failure or final version
    }

    [StructLayout(LayoutKind.Sequential)]
    public class ParseStatus
    {
        public long lStatBits;
        public long lError;
        public long lDocCount;
        public long lTemp_C;
        public long lTemp_F;
        public long lFormLen;
        public String type;
        public String chFormLength;
        public String chTemperature;
        public String chDocCount;
        public String chError;
        public String chStatBits;
        public String version;
        public String ver;
        public String ver_date;
    }
 
    public static class paUtils
    { 

        [DllImport("PADLL")]
        public static extern Int32 paSetBinByBinName(
            [MarshalAs(UnmanagedType.LPStr)] String pBinName);

        [DllImport("PADLL")]
        public static extern Int32 paSetOrientation(
            short orientationID);
         
        [DllImport("PADLL")]
        public static extern Int32 paSetSheetSize(
            [MarshalAs(UnmanagedType.LPArray)] float[] width,
            [MarshalAs(UnmanagedType.LPArray)] float[] length);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetProgress(
            [MarshalAs(UnmanagedType.LPStr)] String ProgressType,
            [MarshalAs(UnmanagedType.LPArray)] byte[] ResultString,
            Int32 ResultStringLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] ResultBufferRead);

        // Note that we have to marshall the pointer to string to padll
        // as StringBuilder. This is because the dll will fill in the string
        // and StringBuilder will be able to convert it to a type safe string.
        // The printer name length is marshaled as an array of 1 element.
        // This emulates a DWORD *.
        [DllImport("PADLL")]
        public static extern Int32 paNewGetDefaultPrinter(
            [MarshalAs(UnmanagedType.LPStr)] StringBuilder DefaultPrinter,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] PrinterNameLength);

        [DllImport("PADLL")]
        public static extern Int32 paNewGetPrinterPort(
            [MarshalAs(UnmanagedType.LPStr)] String printerName,
            [MarshalAs(UnmanagedType.LPStr)] StringBuilder printerPort);

        [DllImport("PADLL")]
        public static extern Int32 paNewGetPrinterDriver(
            [MarshalAs(UnmanagedType.LPStr)] String printerName,
            [MarshalAs(UnmanagedType.LPStr)] StringBuilder printerDriver);

        // Get verbose status
        [DllImport("PADLL")]
        public static extern Int32 paLMGetStatus(
            Int32 StatusType,
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] StatusBytesRead);

        // Get Printer Driver version
        [DllImport("PADLL")]
        public static extern bool paLMGetDriverVersion(
            [MarshalAs(UnmanagedType.LPStr)] string PrinterName,
            [MarshalAs(UnmanagedType.LPStr)] StringBuilder PrinterVersion,
            Int32 PrinterVersionBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] VersionBytesRead);
        
        // Get Langauge Monitor version
        [DllImport("PADLL")]
        public static extern Int32 paLMGetLMVersion(
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] StatusBytesRead);

        // Get Langauge Monitor version
        [DllImport("PADLL")]
        public static extern bool paGetDLLVersion(
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] StatusBytesRead);

        // Get addressed status
        [DllImport("PADLL")]
        public static extern Int32 paLMGetAddStatus(
            String AddressedStatusType,
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] StatusBytesRead);

        // Set New Printer
        [DllImport("PADLL")]
        public static extern bool paSetNewPrinter(
            [MarshalAs(UnmanagedType.LPStr)] String NewStatusBuffer);
        
        // RFID Pipe
        [DllImport("PADLL")]
        public static extern Int32 paLMRFIDPrinter(
            [MarshalAs(UnmanagedType.LPArray)] byte[] DataToWrite,
            Int32 NoBytesToWrite,
            [MarshalAs(UnmanagedType.LPArray)] byte[] ResponseBytes,
            Int32 MaxResponseBytesLen,
           [MarshalAs(UnmanagedType.LPArray)] Int32[] NoBytesRead,
            Int32 Timeout);

        // Determine if printer has valid pipe
        [DllImport("PADLL")]
        public static extern bool paLMPtrIsPipe(
            [MarshalAs(UnmanagedType.LPStr)] String NewStatusBuffer);

        // RFID Pipe (Using Printer Name)
        [DllImport("PADLL")]
        public static extern Int32 paLMPtrRFIDPrinter(
            [MarshalAs(UnmanagedType.LPStr)] String NewStatusBuffer,
            [MarshalAs(UnmanagedType.LPArray)] byte[] DataToWrite,
            Int32 NoBytesToWrite,
            [MarshalAs(UnmanagedType.LPArray)] byte[] ResponseBytes,
            Int32 MaxResponseBytesLen,
           [MarshalAs(UnmanagedType.LPArray)] Int32[] NoBytesRead,
            Int32 Timeout);

        // Get verbose status (Using Printer Name)
        [DllImport("PADLL")]
        public static extern Int32 paLMPtrGetStatus(
            [MarshalAs(UnmanagedType.LPStr)] String NewStatusBuffer,
            Int32 StatusType,
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] StatusBytesRead);

        // Simple (one event) status requests
        [DllImport("PADLL")]
        public static extern Int32 paLMGetPrinterOKStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetPrinterOKStatus2([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetManDeselectStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetOutOfPaperStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetLowPaperStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetJamStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetTicketNotTakenStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetHeadLeverOpenStatus([In, Out] SimpleStatusBool SB);

        [DllImport("PADLL")]
        public static extern Int32 paLMGetPrintInProgress([In, Out] SimpleStatusBool SB);
        
        [DllImport("PADLL")]
        public static extern bool paLMSetPipeTimeOut(Int32 PipeTimeOut);

        [DllImport("PADLL")]
        public static extern bool paLMSetStatusTimeOut(Int32 StatusTimeOut);

        [DllImport("PADLL")]
        public static extern bool paLMSetRegistryKey(
            [MarshalAs(UnmanagedType.LPStr)] String RegistryKeyName,
            Int32 RegistryValue,
            [MarshalAs(UnmanagedType.LPStr)] StringBuilder pResultString,
            Int32 ResultStringLength,
            [MarshalAs(UnmanagedType.LPArray)] Int32[] ResultBufferRead);

        [DllImport("PADLL")]
        public static extern Int32 paSetPrinterKeyByName(
            [MarshalAs(UnmanagedType.LPStr)] String PrinterRegistryKeyName,
            Int32 RegistryValue);
        
        public static Int32 paLMParseFlashProgress(
            [MarshalAs(UnmanagedType.LPStr)] String ProgressType,
            ref flashProgress fp)
        {
            // Declare 5 strings to hold parsed result
            string[] FlashStatusInfo = new string[5];

            // define which character is seperating fields
            char[] splitter = { ',' };
            
            // Parse the string
            FlashStatusInfo = ProgressType.Split(splitter);

            // Move and convert to return structure
            fp.state = Convert.ToInt32(FlashStatusInfo[0]);
            fp.max = Convert.ToInt32(FlashStatusInfo[1]);
            fp.current = Convert.ToInt32(FlashStatusInfo[2]);
            fp.PassFail = FlashStatusInfo[3];
            fp.note = FlashStatusInfo[4];

            return 0;
}

        private const int StartIndex = 0;
        private const int Index_Field00 = 3;    // Bitwise status (Addressed status info)
        private const int Length_Field00 = 2;

        private const int Index_Field01 = 7;    // Error
        private const int Length_Field01 = 3;

        private const int Index_Field02 = 12;   // Ticket count (Version if Extended status)
        private const int Length_Field02 = 8;

        private const int Index_Field03 = 22;   // PH temp
        private const int Length_Field03 = 3;

        private const int Index_Field04 = 27;   // Binary form lengh
        private const int Length_Field04 = 2;

        private const int Index_Field05 = 31;   // Version (Count Byte)
        private const int Index_Field05_EXTENDED = 12;   // Extended status
        private const int Length_Field05 = 0;   // Variable length

        // The PADLL.DLL paParseStatus is not a candidate for 
        // making a bridge from managed to unmanged code. It
        // is safer and better to perform the parsing locally
        // without calling the DLL.
        //
        // Status type must be SNEC or A
        public static Int32 paParseStatus(
            [MarshalAs(UnmanagedType.LPArray)] byte[] StatusBuffer,
            Int32 StatusBufferLength,
            ref ParseStatus pStatus)
        {
            Int32 i;
            Int32 FieldNumber;

            byte[] RawStatusBuffer = new byte[StatusBufferLength];

            // Move the data into a managed array
            for (i = 0; i < StatusBufferLength; i++ )
                RawStatusBuffer.SetValue(StatusBuffer[i], i);

            // Init all the fields to zero
            pStatus.lStatBits = 0x00;
            pStatus.lError = 0;
            pStatus.lDocCount = 0;
            pStatus.lTemp_C = 0;
            pStatus.lTemp_F = 0;
            pStatus.lFormLen = 0;
            pStatus.chDocCount = "";
            pStatus.type = "";
            pStatus.chFormLength = "";
            pStatus.chTemperature = "";
            pStatus.chDocCount = "";
            pStatus.chError = "";
            pStatus.chStatBits = "";
            pStatus.version = "";
            pStatus.ver = "";
            pStatus.ver_date = "";

            switch ((byte)RawStatusBuffer.GetValue(StartIndex))
            {
                // Field 00
                case (byte)'S':
                    // Convert field 0 and return
                    pStatus.type = "S";
                    paParseBuildField00(RawStatusBuffer, ref pStatus, Index_Field00, Length_Field00);
                    break;

                // Field 00 and 01
                case (byte)'N':
                    pStatus.type = "N";
                    paParseBuildField00(RawStatusBuffer, ref pStatus, Index_Field00, Length_Field00);
                    paParseBuildField01(RawStatusBuffer, ref pStatus, Index_Field01, Length_Field01);
                    break;

                // Field 00, 01 and 05
                case (byte)'E':
                    pStatus.type = "E";
                    paParseBuildField00(RawStatusBuffer, ref pStatus, Index_Field00, Length_Field00);
                    paParseBuildField01(RawStatusBuffer, ref pStatus, Index_Field01, Length_Field01);
                    paParseBuildField05(RawStatusBuffer, ref pStatus, Index_Field05_EXTENDED, Length_Field05);
                    break;

                // Field 00 through 05
                case (byte)'C':
                    pStatus.type = "C";
                    paParseBuildField00(RawStatusBuffer, ref pStatus, Index_Field00, Length_Field00);
                    paParseBuildField01(RawStatusBuffer, ref pStatus, Index_Field01, Length_Field01);
                    paParseBuildField02(RawStatusBuffer, ref pStatus, Index_Field02, Length_Field02);
                    paParseBuildField03(RawStatusBuffer, ref pStatus, Index_Field03, Length_Field03);
                    paParseBuildField04(RawStatusBuffer, ref pStatus, Index_Field04, Length_Field04);
                    paParseBuildField05(RawStatusBuffer, ref pStatus, Index_Field05, Length_Field05);
                    break;

                case (byte)'A':
                    pStatus.type = "A";
                    // Need to get the field number to determine what parse method
                    // to call. The field number
                    FieldNumber = ((byte)RawStatusBuffer.GetValue(1) - 0x30) * 10;
                    FieldNumber += ((byte)RawStatusBuffer.GetValue(2) - 0x30);
                    switch (FieldNumber)
                    {
                        // The data to parse always starts at index 3
                        // A##[Data to Parse]
                        case 0:
                            paParseBuildField00(RawStatusBuffer, ref pStatus, 3, Length_Field00);
                            break;
                        case 1:
                            paParseBuildField01(RawStatusBuffer, ref pStatus, 3, Length_Field01);
                            break;
                        case 2:
                            paParseBuildField02(RawStatusBuffer, ref pStatus, 3, Length_Field02);
                            break;
                        case 3:
                            paParseBuildField03(RawStatusBuffer, ref pStatus, 3, Length_Field03);
                            break;
                        case 4:
                            paParseBuildField04(RawStatusBuffer, ref pStatus, 3, Length_Field04);
                            break;
                        case 5:
                            paParseBuildField05(RawStatusBuffer, ref pStatus, 3, Length_Field05);
                            break;
                        default:
                            break;
                }
                    break;
                default:
                    break;
            }

            return 0;
        }

        // Build the bit mapped status bit fields
        private static void paParseBuildField00(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[1024];

            tempBuffer = new byte[Length_Field];
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field, tempBuffer, 0, Length_Field);
            pStatus.lStatBits = (byte)tempBuffer.GetValue(0) << 8;
            pStatus.lStatBits |= (byte)tempBuffer.GetValue(1);
            pStatus.chStatBits = String.Format("0x{0:X4}", pStatus.lStatBits);
        }

        // Build the error bit fields
        private static void paParseBuildField01(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[64];

            tempBuffer = new byte[Length_Field];
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field, tempBuffer, 0, Length_Field);
            pStatus.lError = ((byte)tempBuffer.GetValue(0) - 0x30) * 100;
            pStatus.lError += ((byte)tempBuffer.GetValue(1) - 0x30) * 10;
            pStatus.lError += ((byte)tempBuffer.GetValue(2) - 0x30);
            pStatus.chError = String.Format("{0:D3}", pStatus.lError);
        }

        // Document count
        private static void paParseBuildField02(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[64];

            tempBuffer = new byte[Length_Field];
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field, tempBuffer, 0, Length_Field);

            pStatus.lDocCount = ((byte)(tempBuffer.GetValue(0)) - 0x30) * 10000000;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(1)) - 0x30) * 1000000;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(2)) - 0x30) * 100000;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(3)) - 0x30) * 10000;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(4)) - 0x30) * 1000;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(5)) - 0x30) * 100;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(6)) - 0x30) * 10;
            pStatus.lDocCount += ((byte)(tempBuffer.GetValue(7)) - 0x30);
            pStatus.chDocCount = String.Format("{0:D8}", pStatus.lDocCount);
        }

        // PH Temperature Centigrade and Farenheit
        private static void paParseBuildField03(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[64];

            tempBuffer = new byte[Length_Field];
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field, tempBuffer, 0, Length_Field);
            // Binary value to lTemp_C
            pStatus.lTemp_C =  ((byte)(tempBuffer.GetValue(0)) - 0x30) * 10;
            pStatus.lTemp_C += ((byte)(tempBuffer.GetValue(1)) - 0x30);
            pStatus.lTemp_F = ((pStatus.lTemp_C * 9) / 5) + 32;
            pStatus.chTemperature = String.Format("{0:D2}C", pStatus.lTemp_C);
        }

        // Form length
        private static void paParseBuildField04(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[64];

            tempBuffer = new byte[Length_Field];
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field, tempBuffer, 0, Length_Field);
            // Binary value to lTemp_C
            pStatus.lFormLen = (byte)(tempBuffer.GetValue(0)) * 256;
            pStatus.lFormLen |= (byte)(tempBuffer.GetValue(1));
            if (pStatus.lFormLen == 0)
                pStatus.chFormLength = "Roll";
            else
                pStatus.chFormLength  = String.Format("{0:N2}", ((float)pStatus.lFormLen/1000));
        }

        // Version Note that version is a variable length field.
        // The first 2 characters (Ascii) represent the length of the string
        private static void paParseBuildField05(byte[] RawStatusBuffer, ref ParseStatus pStatus, int Index_Field, int Length_Field)
        {
            byte[] tempBuffer = new byte[64];
            char[] versionBuffer = new char[64];
            int VersionLength;

            // Version is a variable length field so first get the length
            VersionLength = ((byte)(RawStatusBuffer.GetValue(Index_Field)) - 0x30) * 10;
            VersionLength += ((byte)(RawStatusBuffer.GetValue(Index_Field + 1)) - 0x30);

            // Declare a temp buffer of this length
            tempBuffer = new byte[VersionLength];

            // Copy the version portion of the string to tempBuffer
            Array.ConstrainedCopy(RawStatusBuffer, Index_Field + 2, tempBuffer, 0, VersionLength);

            // Resize the version buffer
            versionBuffer = new char[VersionLength];

            // Convert the data from byte to char 
            Encoding ascii = Encoding.ASCII;  // Declare an instance of the encoder
            versionBuffer = ascii.GetChars(tempBuffer);            
            
            // Copy to the version buffer and get the substrings
            pStatus.version = new string(versionBuffer);
            pStatus.ver_date = pStatus.version.Substring(pStatus.version.IndexOf("("));
            pStatus.ver = pStatus.version.Substring(0, pStatus.version.IndexOf("("));
        }

    
    }

    public static class paFunctions
    {
        public static string FormatReturnCode(Int32 ReturnCode)
        {
            switch (ReturnCode)
            {
                case 0:
                    return "PA_ERR_OK";
                case 0xc000 + 1:    // PA_ERR_BAD_PRINTER_NAME
                    return "Could not open current printer";
                case 0xc000 + 10:   // PA_ERR_NO_PRINTER_CHANGE
                    return "Printer key was not set";
                case 0xc000 + 23:
                    return "Form size set to closest possible formsize";
                case 0xc000 + 25:
                    return "PA_ERR_PIPE_OPEN_BUSY";
                case 0xc000 + 26:
                    return "PA_ERR_PIPE_OPEN_FAILED";
                case 0xc000 + 27:
                    return "PA_ERR_PIPE_TIMEOUT";
                case 0xc000 + 28:
                    return "PA_ERR_PRINTER_NOT_RESPONDING";
                case 0xc000 + 29:
                    return "PA_ERR_STATUS_TIMEOUT";
                case 0xc000 + 41:    // Case PA_ERR_UNKNOWN_KEY
                    return "Unknown printer key";
                case 0xffff:
                    return "minus 1";
                default:
                    return "Unknown";
            }
        }
    }

    public class PAConstants
    {
        public const short PA_ERR_OK = 0;
        public const int PA_ERR = 49152;
        public const int PA_ERR_BAD_PRINTER_NAME = (PA_ERR + 1);
        public const int PA_ERR_BAD_PORT_NAME = (PA_ERR + 2);
        public const int PA_ERR_SYSTEM = (PA_ERR + 3);
        public const int PA_ERR_VAL_NOT_SET = (PA_ERR + 4);
        public const int PA_ERR_STRING_TOO_LONG = (PA_ERR + 5);
        public const int PA_ERR_PARAM1 = (PA_ERR + 6);
        public const int PA_ERR_PARAM2 = (PA_ERR + 7);
        public const int PA_ERR_PARAM3 = (PA_ERR + 8);
        public const int PA_ERR_PARAM4 = (PA_ERR + 9);
        public const int PA_ERR_NO_PRINTER_CHANGE = (PA_ERR + 10);
        public const int PA_ERR_ACCESS = (PA_ERR + 11);
        public const int PA_ERR_NO_PRINTERS = (PA_ERR + 12);
        public const int PA_ERR_REASSIGN = (PA_ERR + 13);
        public const int PA_ERR_BLOWN_POINTER = (PA_ERR + 14);
        public const int PA_ERR_OS_UNSUPPORTED = (PA_ERR + 15);
        public const int PA_ERR_PORT_BUSY = (PA_ERR + 16);
        public const int PA_ERR_BAD_STAT_GROUP = (PA_ERR + 17);
        public const int PA_ERR_BAD_STAT_FIELD = (PA_ERR + 18);
        public const int PA_ERR_BUFFER_TOO_SMALL = (PA_ERR + 19);
        public const int PA_ERR_INSUFFICIENT_MEM = (PA_ERR + 20);
        public const int PA_ERR_UNKNOWN_PRINTER = (PA_ERR + 21);
        public const int PA_ERR_PORT_HAS_SPOOLED_JOB = (PA_ERR + 22);
        public const int PA_ERR_SHEET_NOT_FOUND = (PA_ERR + 23);
        public const int PA_ERR_BIN_NOT_FOUND = (PA_ERR + 24);
        //Error codes for language monitor 
        public const int PA_ERR_PIPE_OPEN_BUSY = (PA_ERR + 25);
        public const int PA_ERR_PIPE_OPEN_FAILED = (PA_ERR + 26);
        public const int PA_ERR_PIPE_TIMEOUT = (PA_ERR + 27);
        public const int PA_ERR_PRINTER_NOT_RESPONDING = (PA_ERR + 28);
        public const Int32 PA_ERR_STATUS_TIMEOUT = (PA_ERR + 29);

        // Error codes related to firmware reprogramming 
        public const int PA_ERR_PRINTER_NOT_READY = (PA_ERR + 31);
        public const int PA_ERR_BIN_FILE_NOT_FOUND = (PA_ERR + 32);
        public const int PA_ERR_BIN_FILE_BAD_EXTENTION = (PA_ERR + 33);
        public const int PA_ERR_CAN_NOT_GET_MODEL = (PA_ERR + 34);
        public const int PA_ERR_STATUS_READ_ERROR = (PA_ERR + 35);
        public const int PA_ERR_INCORRECT_BIN_FILE = (PA_ERR + 36);
        public const int PA_ERR_FLASH_NOT_ENTERED = (PA_ERR + 37);
        public const int PA_ERR_FLASH_BAD_BIN_FILE = (PA_ERR + 38);
        public const int PA_ERR_FLASH_REPROGRAM_FAILED = (PA_ERR + 39);
        public const int PA_ERR_MODEL_NOT_FOUND = (PA_ERR + 40);
        public const int PA_ERR_UNKNOWN_KEY = (PA_ERR + 41);

        ///''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
        // Defines for legal paper bins ticket printer series 
        ///''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' 
        public const string PA_CUT_ALL = "Cut on all pages";
        public const string PA_CUT_2ND = "Cut every 2nd page";
        public const string PA_CUT_3RD = "Cut every 3rd page";
        public const string PA_CUT_4TH = "Cut every 4th page";
        public const string PA_CUT_5TH = "Cut every 5th page";
        public const string PA_CUT_LAST = "Cut on last page only";
        public const string PA_NO_CUT = "Disable cutting";

        public const int PA_MAX_BUFFER_SIZE = 1024;
        public const int DEFAULT_PIPE_TIMEOUT = 20000;
        public const int DEFAULT_STATUS_TIMEOUT = 20000;

        // PA status select constants 
        public const int PA_STAT_SHORT = 0;
        public const int PA_STAT_NORMAL = 1;
        public const int PA_STAT_EXTENDED = 2;
        public const int PA_STAT_COUNT = 3;
        public const int PA_STAT_TEMPERATURE = 4;
        public const int PA_STAT_FORMLENGTH = 5;
        public const int PA_STAT_COMPLETE = 6;
        public const int PA_STAT_LAST = 7;
        public const int PA_STAT_DOC_IN_PROGRESS = 8;

        // PA mask for bitwise status bits (16 bit form)
        public const int STAT16_SYSTEM_NOT_READY   = 0x8000;  // System Not Ready
        public const int STAT16_DIAG_ERROR = 0x4000;          // Sys/Diag Error
        public const int STAT16_MANUAL_DESELECT = 0x2000;     // Manual Deselect
        public const int STAT16_OUTPUT_JAM = 0x1000;          // Output Jam
        public const int STAT16_REGISTRATION_ERROR = 0x0800;  // Paper Reg. Error
        public const int STAT16_CUTTER_JAM = 0x0400;          // Cutter Jam
        public const int STAT16_HEAD_LEVER_UP = 0x0200;       // Head Lever Up
        public const int STAT16_OUT_OF_PAPER = 0x0100;        // Out of Paper

        public const int STAT16_DOC_IN_PROGRESS = 0x0080;     // Doc in Process
        public const int STAT16_DATA_ERROR = 0x0040;          // Data 0 = data error, 1 = purge occurred
        public const int STAT16_BUSY = 0x0020;                // Busy
        public const int STAT16_TEMP_WAIT = 0x0010;           // PH Temp Wait
        public const int STAT16_DOC_IN_PRESENTER = 0x0008;    // Ticket Not Taken
        public const int STAT16_AVG_POWER_WAIT = 0x0004;      // Avg. Power Wait
        public const int STAT16_DISPOSE_TOGGLE = 0x0002;      // Dispose Toggle Flag
        public const int STAT16_LOW_PAPER = 0x0001;           // Low Paper

        // RFID Constants
        public const int RFID_READER_NOT_DETERMINED = 0x30;   // Reader presence unknown
        public const int RFID_NO_READER = 0x31;               // No reader installed
        public const int RFID_READER_NON_FUNCTIONAL = 0x32;   // Reader installed, but not functional
        public const int RFID_NO_STOCK = 0x33;                // Reader present, stock is OOP
        public const int RFID_TAGLESS_STOCK = 0x34;           // Non-RFID stock present.
        public const int RFID_STOCK_LOADED = 0x35;            // RFID stock present, but not ready
        public const int RFID_TAG_READY = 0x36;               // RFID Tag in position for reading/writing
        public const int RFID_TAG_FAULTY = 0x37;              // cannot read a tag which should be there

        public const int RFID_SUCCESS = 0x30;                 // Operation complete; command specific data may follow
        public const int RFID_BUSY = 0x31;                    // Operation still in process
        public const int RFID_FAILURE = 0x32;                 // Operation failed
        public const int RFID_CANCELLING = 0x33;              // A cancel command is in process
        public const int RFID_CANCELLED = 0x34;               // A cancel operation is complete
        public const int RFID_NO_STATUS = 0x35;               // No operation is in process for this query
        public const int RFID_INVALID_COMMAND = 0x36;         // Command and/or parameters invalid
    }
}
